// LIBRERIA CON LE FUNZIONI PER COLORARE IL TESTO SOTTO LINUX
//
// (C) CTIO & Simone Gasparini some rights reserved


#ifndef ctio_header
#define ctio_header

#ifdef __cplusplus
extern "C" {
#endif


// INIZIALIZZAZIONE DI CTIO
int init_ctio();


// CODICI DI ERRORE INIZIALIZZAZIONE
enum { CANNOT_INIT_MOUSE   = 1, 
       CANNOT_INIT_KBD     = 2, 
       CANNOT_INIT_DISPLAY = 4 };

/* Pass a nonzero error from init_ctio to this func for a
 descriptive explanation */
const char *ctio_error_str( int error );

// DISIBILITA LA FUNZIONE DEI TASTI CTRL+C
void disable_ctrl_c_interrupt();

// UNINIZIALIZZAZIONE DI CTIO
void uninit_ctio();


/************************************************************************\
 *                                                                      *
 * CTIO types                                                           *
 *                                                                      *
\************************************************************************/

#include <sys/types.h>


// There is a global variable of this type called "event" which is filled by the get_input() function.
typedef struct {

   ushort key;    /* in the case of a mouse event, key == MOUSE_EVENT */
   ushort kbstat; /* kbstat is filled for mouse events as well */ 

   unsigned char x, y;     /* mouse position */
   ushort type;            /* mouse event type (0 if keyboard event) */
   unsigned char buttons;  /* which buttons are down */
   }
   ct_event;


// A clr_t contains the display attributes of a character cell.
// For portability I recommend you compose it using the macro
// CLR( fg, bg ) defined below. If the character in question is
// a box-drawing character, the resulting clr_t should be ORed
// with BOX_BIT, or else it will not display properly under Linux.
typedef unsigned char clr_t;


// Each cell on the screen is represented by a single clrchar.
// You can access the screen_buf array directly and change the 
// character or attribute in any cell, then call refresh_vt()
// to display the changes.
typedef struct {

   char ch;
   clr_t clr;
   }
   clrchar;


// This is a rectangular array of clrchars. screen_buf is a ct_image,
// so you can find the current screen width and height by examining
// screen_buf.w and screen_buf.h. You can create a ct_image using 
// ct_create() and copy it to screen_buf using ct_copy.
typedef struct {

   int x, y;          /* position relative to screen top left ( 0, 0 ) */
   int w, h;          /* width, height */
   int size;          /* array size ( w * h ) */
   int scroll_top;    /* top line of scroll region */
   int scroll_bottom; /* bottom line of scroll region */
   clrchar blank;     /* used for functions which clear areas */
   clrchar *img;      /* array of clrchars */
   }
   ct_image;


// ######################################################## CTIO INPUT


/* This global variable is filled by get_input()
 * all events are recorded in it
 */
extern ct_event event;


/* This is the only function for reading the mouse and keyboard.
 * It waits until an event occurs before returning.
 * See the README for how to interpret events.
 */
ushort get_input(); 


/* Call this to check if an event is ready for reading.
 * See the note on key_awaits(), below.
 */
#define event_awaits() ( mouse_awaits() || key_awaits() )


// ######################################################## CTIO OUTPUT

/* text colour attributes 
 * ======================
 */

/* All 0-15 are valid as FG colours; 
 * Only 0-7 are valid BG colours.
 * use the CLR( fg, bg ) macro to avoid errors
 */

#define CBLACK         0
#define CRED           1
#define CGREEN         2
#define CBROWN         3
#define CBLUE          4
#define CMAGENTA       5
#define CCYAN          6
#define CLIGHTGRAY     7
#define CDARKGRAY      8
#define CLIGHTRED      9
#define CLIGHTGREEN   10
#define CYELLOW       11
#define CLIGHTBLUE    12
#define CLIGHTMAGENTA 13
#define CLIGHTCYAN    14
#define CWHITE        15

/* The box bit must be set in the colour of all box 
 * characters or they will not be drawn as box chars 
 * under Linux.
 */
#define BOX_BIT 128

/* ORing this with a clr_t makes the fg bright
 */
#define CT_BRIGHT  8

/* Build your clr_t with this to avoid errors which would be 
 * caused if the high bit (BOX_BIT) is accidentally set.
 */
#define CLR( fg, bg ) ( ((clr_t)fg & 0xF) | (((clr_t)bg & 0x7) << 4) )

/* Extract the fg or bg from a clr_t with these
 */
#define FGCLR( clr ) ( (clr_t)clr & 0xF )
#define BGCLR( clr ) ( ((clr_t)clr >> 4) & 0x7 )

/* Change the fg or bg of clr in place with these
 */
#define CHANGE_FG( clr, fg ) { clr = ( clr & 0xF0 ) | fg; }
#define CHANGE_BG( clr, bg ) { clr = ( clr & 0x0F ) | ( bg << 4 ); }


/* ct_image manipulation
 * =====================
 */


/* This allocates a rectangular array of clrchar and sets x, y, 
 * w, h and all other fields to sane values. The malloced buffer 
 * must be freed by the caller.
 */
ct_image ct_create( int x, int y, int w, int h, clrchar filler );


/* ct_copy copies the overlap based on the x, y, w and h of the
 * two images, from src to dst.
 */
int ct_copy( ct_image *src, ct_image *dst );


/* Fill a ct_image with its blank member
 */
int ct_clear( ct_image *image );


/* Fill a ct_image with a given clrchar
 */
int ct_fill( ct_image *image, clrchar blank );


/* Shuffle everything up or down and insert new lines.
 * Use a negative value for numlines to scroll up.
 * Pass a NULL for new_lines if you want blank lines
 * inserted.
 */
int ct_scroll( ct_image *image, int numlines, clrchar *new_lines );


/* Set the top and bottom lines to included in scrolls 
 * Not to be used on screen_buf; use the vt_ version instead.
 */
int  ct_set_scroll_region( ct_image *image, int top, int bottom );


/* x and y are relative to the top left of the dest image 
 * Text will be clipped to fit.
 */
int ct_text_xy( ct_image *dest, int x, int y, 
                const char *text, clr_t colour );


/* This is a multi-colour, tab-aware text_xy function.
 * slength is the length of the char * src.
 * colours is an array of clr_t which must be at least as
 * long as slength.
 * if slength is 0, the line will be cleared and src, 
 * srcclr and tab_width can also be 0.
 */ 
int ct_clrtext_xy( ct_image *target, int x, int y,
                   char *src, clr_t *srcclr, int slength,
                   int tab_width );


/* use this to check if the mouse is over a ct_image 
 */
int ct_under( ct_image *area, int x, int y );


/* Draw a border around the inside edge of a ct_image
 */
int ct_border( ct_image *box, 
               const char *border_style, 
               clr_t border_clr );


/* border styles for passing to draw_ct_border()
 * single or double lines
 */
extern const char *s_border;
extern const char *d_border;

/* Indices to s_border and d_border.
 * eg. s_border[TLCNR] is the single-line, top-left 
 * corner character
 */
enum { VBAR, HBAR, TLCNR, TRCNR, BLCNR, BRCNR, LTEE, RTEE, UPRTEE, LWRTEE, CROSS };



/* screen output
 * =============
 */

/* This is the clrchar which is output in place of control 
 * characters in the ranges 0-31 and 127-159. By default it
 * is a white 'X' on a red background. If you change it, I
 * recommend you choose a normal, printable character.
 */
extern clrchar replacement_char;


/* To display a ct_image, ct_copy it to screen_buf and 
 * call refresh_vt() to show it 
 */
extern ct_image screen_buf;   


/* Transfer clrchars from screen_buf to console screen.
 */
int  vt_refresh(); /* scroll region only */
int  vt_refresh_lines( int first, int last ); 
int  vt_refresh_line( int line ); 


/* Scroll speed can be improved under Linux by scrolling 
 * console screen and refreshing only the new lines.
 */
int  vt_scroll( int numlines, clrchar *new_lines );


/* Set the top and bottom line of the console scroll region.
 * Only full-width scrolling is possible.
 */ 
int  vt_set_scroll_region( int top_line, int bottom_line );


/* Cursor Display and Positioning 
 */
void hide_mouse_cursor();
void show_mouse_cursor();

void hide_text_cursor();
void show_text_cursor();
void put_text_cursor( int col, int row ); /* Zero-based */

/* Stores cursor pos in *col and *row and returns cursor 
 * visibility flag. Cursor is visible if flag is set AND col 
 * and row are within the screen area.
 */
int text_cursor_pos( int *col, int *row );



// ######################################################## SPECIAL

/* This is for stripping end-of-line characters from buffers
 * filled by fgets. It removes the '\n' from lines read from
 * UNIX files and the '\r\n' from lines read from DOS files.
 * I provide it because failure to strip these characters
 * causes garbled screen output.
 */
int remove_eol( char *linebuf );


// ######################################################## MIE FUNZ

int iniz_screen();

#ifdef __cplusplus
}
#endif


#endif /* ndef ctio_header */
